==> sv_v0.13.tgz: supervisor/obs_massdimm.tcl <==
## MASS-DIMM observations scenario. To be started from SV v0.12 or later.
# See description in obs_massdimm.txt and in obs_massdimm.fig.

#foreach p {CENT CIRC TLSP OBJM MASS DIMM CORR SV cmd is_cmd wait_cmd} {
#	proc $p args {return 1}}
#proc add_log {msg} {puts $msg}
########################################################################
# Global control variables:

set BG_NEED 0
set CENTERED 0
set TEND 0
set MEASURING 0
set OVER 0
set WTIME 0

# Set telescope coordinates:
cmd TLSP set longitude="[CIRC longitude]" latitude="[CIRC latitude]"
########################################################################
# Import the configuration file parameters on place, to release the 
# CFG errors immediately
set wait_time [SV wait_time]
set n_try [CENT n_try]
set work_zone [CENT work_zone]
set cent_zone [CENT cent_zone]
set alpha [CENT alpha]
set cent_period [CENT period]
set shift_step [TLSP shift_step]
set shift_max [TLSP shift_max]
set bkgr_shift_ra [TLSP bkgr_shift_ra]
set bkgr_shift_dec [TLSP bkgr_shift_dec]
########################################################################
# Procedures:

# Getting the new object 
# Affects: OBJM(RA), OBJM(DEC), OVER and TEND
proc GETOBJECT {} {
	add_log "GETOBJECT"
	global OVER TEND
	# normally we don't need to change telescope orientation...
	set OVER 0
	# select object and return its equatorial coods, best visibility time
	# and current hour angle in 
	cmd OBJM run ra dec tvis hourh
	# If there is no object - nothing to observe!:
	if {[OBJM status]=="NOSTAR"} {
		set TEND 0
		return
	}
	cmd TLSP set ra="[OBJM ra]" dec="[OBJM dec]"
	cmd TLSP get tvis tvis2
	set teltvis [TLSP tvis]
	# ... But we still should check the possibilities:
	if {[OBJM tvis] > [TLSP tvis] && [TLSP tvis2] > [TLSP tvis]} {
		set OVER 1
		set teltvis [TLSP tvis2]
	} 
	if {$teltvis>[OBJM tvis]} {
		set tvis [OBJM tvis]
	} else {
		set tvis $teltvis
	}
	set TEND [expr [clock seconds]+$tvis]
}

# Pointing the telescope: assumes that the coordinates are already preset:
# Affects: nothing
proc SLEWTEL {over} {
	add_log "SLEWTEL"
	global TEND
	if {$over} {
		cmd TLSP run over
	} else {
		cmd TLSP run
	}
	if {[TLSP status]!="READY"} {
		set TEND 0
	}
}

# Returns the next signs of shifts along the spiral:
# @param init 1 for initialization of spiral movement
# @return [list RA-shift-sign DEC-shift-sign]
set sp_step 0
set sp_rad 0
proc get_spiral {{init 0}} {
	global sp_step sp_rad
	if {$init} {
		set sp_step 0
		set sp_rad 0
		return [list 0 0]
	} else {
		if {$sp_step==8*$sp_rad} {
			incr sp_rad
			set sp_step 0
			return [list 1 0]
		} else {
			set st [expr ($sp_step/$sp_rad) % 8]
			set l [\
			switch $st {
				0 - 7 {list 0 1} 1 - 2 {list -1 0} 3 - 4 {list 0 -1} \
				5 - 6 {list 1 0}
			}]
			incr sp_step
			return $l
		}
	}
}

# Searching the star around pointing position
# Affects: CENTERED
proc SEARCH {} {
	global CENTERED sp_rad shift_step shift_max
	add_log "SEARCH"
	set CENTERED 0
	GUIDER
	# Initialize spiral:
	get_spiral 1
	while {"[CENT status]"=="NOSTAR" && 
		[expr $sp_rad*$shift_step]<=$shift_max} {
		if {$sp_rad} {
			set shra [expr [lindex $shifts 0]*$shift_step]
			set shdec [expr [lindex $shifts 1]*$shift_step]
			cmd TLSP run dra=$shra ddec=$shdec
		}
		GUIDER
		add_log "Centered=$CENTERED"
		# Get the current shift along spiral
		set shifts [get_spiral]
	}
}

# Determine the period of background taking by circumstances [seconds]:
proc get_bkgr_period {} {
	cmd CIRC get hsun hmoon
	if {[CIRC hsun]>-20 || [CIRC hmoon]>5} {
		return 300
	} else {
		return 600
	}
}
# Measuring background with telescope shift
# Affects: MEASURING, BG_NEED
proc BACKGROUND {} {
	global MEASURING BG_NEED bkgr_shift_ra bkgr_shift_dec
	add_log "BACKGROUND"
	set MEASURING 0
	cmd TLSP run dra=$bkgr_shift_ra ddec=$bkgr_shift_dec
	cmd MASS run bkgr
	set BG_NEED 0
	set shra [expr $bkgr_shift_ra*(-2)]
	set shdec [expr $bkgr_shift_dec*(-2)]
	cmd TLSP run dra=$shra ddec=$shdec
	cmd TLSP run dra=$bkgr_shift_ra ddec=$bkgr_shift_dec
	after [expr int([get_bkgr_period]*1000)] {set BG_NEED 1}
	return
}

# Iterative centering the star
# Uses: MEASURING
# Affects: CENTERED
# @param auto flag of auto-restart with help of after-script. 
# NOTE: call as "GUIDER" if you want it working nonwithstanding MEASURING=0.
# after-script must call it as "GUIDER 1"
proc GUIDER {{auto 0}} {
	global MEASURING n_try cent_zone work_zone alpha cent_period CENTERED
	if {!$auto} { 
		add_log "GUIDER" 
	} else {
		if {!$MEASURING} return
	}
	set try 0
	cmd CENT get dra ddec
	while {[CENT status]!="NOSTAR" && ($MEASURING || !$auto) && \
			([expr abs([CENT dra])]>$cent_zone || \
			[expr abs([CENT ddec])]>$cent_zone) && \
			$try<$n_try} {
		set corra [expr (-1)*[CENT dra] * $alpha]
		set cordec [expr (-1)*[CENT ddec] * $alpha]
		cmd CORR run dra=$corra ddec=$cordec
		cmd CENT get dra ddec
		incr try
	}
	set out [expr abs([CENT dra])>$work_zone || abs([CENT ddec])>$work_zone]
	set CENTERED [expr ![string match -nocase [CENT status] "NOSTAR"] && !$out]
add_log "CENTERED->$CENTERED"
	if {$MEASURING} {
		after [expr int($cent_period*1000.)] { GUIDER 1 }
	}
}

# Main measurements loop piece
# Uses: CENTERED, BG_NEED
set dimmci 0
set massci 0
proc MAINMETER {} {
	global massci dimmci CENTERED BG_NEED
	add_log "MAINMETER"
	wait_cmd $dimmci $massci
	if {![is_cmd $dimmci]} {
		if {$dimmci} {cmd DIMM get data}
		if {$CENTERED} {
			set dimmci [cmd DIMM run &]
		} else {
			cmd DIMM stop now
		}
	}
	if {![is_cmd $massci]} {
		if {$massci} {cmd MASS get data}
		if {$BG_NEED} {
			cmd DIMM stop now
		} elseif {$CENTERED} {
			set massci [cmd MASS run &]
		} else {
			cmd MASS stop now
		}
	}
}

# Compute the time rest for observations, in [sec]
# @param LImiting hour angle in decimal hours
# @retur time rest for observations in seconds
proc getwtime {} {
	global TEND
	if {[set t [expr abs($TEND - [clock seconds])]]>0} {
		return $t
	} else { return 0}
}

# Stop running (or not) detectors
proc stop {} {
	add_log "STOP"
	cmd MASS stop now
	cmd DIMM stop now
}

# Wait some time before "situation becomes better"
# @param dt waiting time
proc wait {} {
	global wait_time
	add_log "WAIT $wait_time seconds..."
	set till [expr [clock seconds] + $wait_time]
	while {[clock seconds]<$till} {after 1000; 	update}
}

########################################################################
## Now the scenario body itself goes:
########################################################################

# Infinite loop of targets observations:
while 1 {
	GETOBJECT

	set WTIME [getwtime]
add_log "WTIME= $WTIME"	
	if {$WTIME<=0} {
		wait
		continue	
	}
	
	SLEWTEL $OVER
	SEARCH	
	set WTIME [getwtime]

	if {$WTIME<=0 || !$CENTERED}  {
		wait
		continue	
	}
	
	set BG_NEED 1
	while {$WTIME>0 && $CENTERED} {
		if {$BG_NEED} {
			BACKGROUND
			set MEASURING 1
			GUIDER
		}
		MAINMETER
		set WTIME [getwtime]
		add_log "WTIME=$WTIME CENTERED=$CENTERED"
	}
	
	set MEASURING 0
	
	stop	
}

==> sv_v0.15.tgz: supervisor/obs_massdimm.tcl <==
## MASS-DIMM observations scenario. To be started from SV v0.12 or later.
## Scenario version: 0.02
## Written  by N.Shatsky, kolja@sai.msu.ru
## See description in obs_massdimm.txt and in obs_massdimm.fig.

#foreach p {CENT CIRC TLSP OBJM MASS DIMM CORR SV cmd is_cmd wait_cmd} {
#	proc $p args {return 1}}
#proc add_log {msg} {puts $msg}
########################################################################
# Global control variables:

set BG_NEED 0
set CENTERED 0
set TEND 0
set MEASURING 0
set OVER 0
set WTIME 0

# Set telescope coordinates:
cmd TLSP set longitude="[CIRC longitude]" latitude="[CIRC latitude]"
########################################################################
# Import the configuration file parameters on place, to release the 
# CFG errors immediately
set wait_time [SV wait_time]
set n_try [CENT n_try]
set work_zone [CENT work_zone]
set cent_zone [CENT cent_zone]
set alpha [CENT alpha]
set cent_period [CENT period]
set shift_step [TLSP shift_step]
set shift_max [TLSP shift_max]
set bkgr_shift_ra [TLSP bkgr_shift_ra]
set bkgr_shift_dec [TLSP bkgr_shift_dec]
########################################################################
# after-scripts identifiers for cancelling:
set background_after 0
set guider_after 0
########################################################################
# Procedures:

# Getting the new object 
# Affects: OBJM(RA), OBJM(DEC), OVER and TEND
proc GETOBJECT {} {
	add_log "GETOBJECT"
	global OVER TEND
	# normally we don't need to change telescope orientation...
	set OVER 0
	# select object and return its equatorial coods, best visibility time
	# and current hour angle in 
	cmd OBJM run ra dec tvis hourh object
	# If there is no object - nothing to observe!:
	if {[OBJM status]=="NOSTAR"} {
		set TEND 0
		return
	}
	cmd TLSP set ra="[OBJM ra]" dec="[OBJM dec]"
	cmd TLSP get tvis tvis2
	set teltvis [TLSP tvis]
	# ... But we still should check the possibilities:
	if {[OBJM tvis] > [TLSP tvis] && [TLSP tvis2] > [TLSP tvis]} {
		set OVER 1
		set teltvis [TLSP tvis2]
	} 
	if {$teltvis>[OBJM tvis]} {
		set tvis [OBJM tvis]
	} else {
		set tvis $teltvis
	}
	set TEND [expr [clock seconds]+$tvis]
	cmd MASS set object=[OBJM object]
	cmd DIMM set object=[OBJM object]
}

# Pointing the telescope: assumes that the coordinates are already preset:
# Affects: nothing
proc SLEWTEL {over} {
	add_log "SLEWTEL"
	global TEND
	if {$over} {
		cmd TLSP run over
	} else {
		cmd TLSP run
	}
	if {[TLSP status]!="READY"} {
		set TEND 0
	}
	cmd TLSP get ra dec
}

# Returns the next signs of shifts along the spiral:
# @param init 1 for initialization of spiral movement
# @return [list RA-shift-sign DEC-shift-sign]
set sp_step 0
set sp_rad 0
proc get_spiral {{init 0}} {
	global sp_step sp_rad
	if {$init} {
		set sp_step 0
		set sp_rad 0
		return [list 0 0]
	} else {
		if {$sp_step==8*$sp_rad} {
			incr sp_rad
			set sp_step 0
			return [list 1 0]
		} else {
			set st [expr ($sp_step/$sp_rad) % 8]
			set l [\
			switch $st {
				0 - 7 {list 0 1} 1 - 2 {list -1 0} 3 - 4 {list 0 -1} \
				5 - 6 {list 1 0}
			}]
			incr sp_step
			return $l
		}
	}
}

# Searching the star around pointing position
# Affects: CENTERED
proc SEARCH {} {
	global CENTERED sp_rad shift_step shift_max
	add_log "SEARCH"
	set CENTERED 0
	GUIDER
	# Initialize spiral:
	get_spiral 1
	while {([CENT dra]==999 || [CENT ddec]==999) && 
		[expr $sp_rad*$shift_step]<=$shift_max} {
		if {$sp_rad} {
			set shra [expr [lindex $shifts 0]*$shift_step]
			set shdec [expr [lindex $shifts 1]*$shift_step]
			cmd TLSP run dra=$shra ddec=$shdec
		}
		GUIDER
		# Get the current shift along spiral
		set shifts [get_spiral]
	}
}

# Determine the period of background taking by circumstances [seconds]:
proc get_bkgr_period {} {
	cmd CIRC get hsun hmoon
	if {[CIRC hsun]>-20 || ([CIRC hmoon]>0 && [CIRC hmoon]<15)} {
		return 600
	} else {
		return 1800
	}
}
# Measuring background with telescope shift
# Affects: MEASURING, BG_NEED
proc BACKGROUND {} {
	global MEASURING BG_NEED bkgr_shift_ra bkgr_shift_dec background_after
	add_log "BACKGROUND"
	set BG_NEED 0
	set MEASURING 0
	STOP
	cmd TLSP run dra=$bkgr_shift_ra ddec=$bkgr_shift_dec
	cmd MASS run bkgr
	set shra [expr $bkgr_shift_ra*(-2)]
	set shdec [expr $bkgr_shift_dec*(-2)]
	cmd TLSP run dra=$shra ddec=$shdec
	cmd TLSP run dra=$bkgr_shift_ra ddec=$bkgr_shift_dec
	set interval [expr int([get_bkgr_period]*1000)]
	set background_after [after $interval {set BG_NEED 1}]
	return
}

# Iterative centering the star
# Uses: MEASURING
# Affects: CENTERED
# @param auto flag of auto-restart with help of after-script. 
# NOTE: call as "GUIDER" if you want it working nonwithstanding MEASURING=0.
# after-script must call it as "GUIDER 1"
proc GUIDER {{auto 0}} {
	global MEASURING n_try cent_zone work_zone alpha cent_period CENTERED
	global guider_after
	if {!$auto} { 
		add_log "GUIDER" 
	} else {
		if {!$MEASURING} return
	}
	set try 0
	cmd CENT get dra ddec
	while {[CENT dra]!=999 && [CENT ddec]!=999 && ($MEASURING || !$auto) && \
			([expr abs([CENT dra])]>$cent_zone || \
			[expr abs([CENT ddec])]>$cent_zone) && \
			$try<$n_try} {
		set corra [expr (-1)*[CENT dra] * $alpha]
		set cordec [expr (-1)*[CENT ddec] * $alpha]
		cmd CORR run dra=$corra ddec=$cordec
		cmd CENT get dra ddec
		incr try
	}
	set CENTERED [expr abs([CENT dra])<=$work_zone && abs([CENT ddec])<=$work_zone]
	cmd TLSP get ra dec tvis
add_log "CENTERED->$CENTERED"
	if {$MEASURING} {
		set guider_after [after [expr int($cent_period*1000.)] { GUIDER 1 }]
	}
}

# Main measurements loop piece
# Uses: CENTERED, BG_NEED
set dimmci -1
set massci -1
proc MAINMETER {} {
	global massci dimmci CENTERED BG_NEED
	add_log "MAINMETER"
	wait_cmd $dimmci $massci
	cmd OBJM get hour zen
	if {![is_cmd $dimmci]} {
		if {$dimmci>=0} {cmd DIMM get data}
		if {$CENTERED && !$BG_NEED} {
			set dimmci [cmd DIMM run &]
		}
	}
	if {![is_cmd $massci]} {
		if {$massci>=0} {cmd MASS get data}
		if {$CENTERED && !$BG_NEED} {
			set massci [cmd MASS run &]
		}
	}
}

# Compute the time rest for observations, in [sec]
# @param Limiting hour angle in decimal hours
# @return time rest for observations in seconds
proc getwtime {} {
	global TEND
	if {[set t [expr $TEND - [clock seconds]]]>0} {
		return $t
	} else { return 0}
}

# Stop running (or not) detectors
proc STOP {} {
	global background_after guider_after
	add_log "STOP"
	cmd MASS stop now
	cmd DIMM stop now
	after cancel $background_after
	after cancel $guider_after
}

# Wait some time before "situation becomes better"
# @param dt waiting time
proc wait {} {
	global wait_time
	add_log "WAIT $wait_time seconds..."
	set till [expr [clock seconds] + $wait_time]
	while {[clock seconds]<$till} {after 1000; 	update}
}

########################################################################
## Now the scenario body itself goes:
########################################################################

# Infinite loop of targets observations:
while 1 {
	GETOBJECT

	set WTIME [getwtime]

	if {$WTIME<=0} {
		wait
		continue	
	}
	
	SLEWTEL $OVER
	SEARCH	
	set WTIME [getwtime]

	if {$WTIME<=0 || !$CENTERED}  {
		wait
		continue	
	}
	
	set BG_NEED 1
	while {$WTIME>0 && $CENTERED} {
		if {$BG_NEED} {
			BACKGROUND
			set MEASURING 1
			GUIDER
			# reset command identifiers for DIMM and MASS to prevent GET DATA:
			set massci [set dimmci -1]
		}
		MAINMETER
		set WTIME [getwtime]
	}
	
	set MEASURING 0
	
	STOP	
}
